@@ -1,8 +1,3 @@
-# coding=utf-8
-from __future__ import (
-    absolute_import, division, print_function, unicode_literals
-)
-
 import logging
 
 import requests
@@ -42,6 +37,11 @@ class CloseIO(object):
 
         return self._api_cache
 
+    @parse_response
+    @handle_errors
+    def me(self):
+        return self._api.me.get()
+
     @parse_response
     @handle_errors
     def get_lead(self, lead_id):
@@ -210,7 +210,7 @@ class CloseIO(object):
             "lead_id": lead_id,
             "assigned_to": assigned_to,
             "text": text,
-            "due_date": due_date.date().isoformat() if due_date else None,
+            "due_date": due_date.isoformat() if due_date else None,
             "is_complete": is_complete
         })
 
@@ -220,6 +220,11 @@ class CloseIO(object):
         fields = convert(fields)
         return self._api.task(task_id).put(fields)
 
+    @parse_response
+    @handle_errors
+    def delete_task(self, task_id):
+        return self._api.task(task_id).delete()
+
     @parse_response
     @handle_errors
     def get_tasks(self, **kwargs):
@@ -230,6 +235,8 @@ class CloseIO(object):
             if isinstance(v, bool)
         })
 
+        kwargs.setdefault('_order_by', '-date_created')
+
         return paginate(
             self._api.task.get,
             **kwargs
@@ -275,7 +282,7 @@ class CloseIO(object):
     @handle_errors
     def create_activity_email(self, **kwargs):
         kwargs.setdefault('status', 'draft')
-        self._api.activity.email.post(kwargs)
+        return self._api.activity.email.post(kwargs)
 
     @parse_response
     @handle_errors
@@ -285,6 +292,19 @@ class CloseIO(object):
             lead_id=lead_id,
         )
 
+    @parse_response
+    @handle_errors
+    def create_activity_call(self, **kwargs):
+        return self._api.activity.call.post(kwargs)
+
+    @parse_response
+    @handle_errors
+    def get_activity_call(self, lead_id):
+        return paginate(
+            self._api.activity.call.get,
+            lead_id=lead_id,
+        )
+
     @parse_response
     @handle_errors
     def get_activity_note(self, lead_id):
@@ -358,7 +378,7 @@ class CloseIO(object):
 
             user.update({
                 key[5:]: value
-                for key, value in membership.iteritems()
+                for key, value in membership.items()
                 if key.startswith('user_')
             })
 
@@ -376,7 +396,7 @@ class CloseIO(object):
             if membership['user_id'] == user_id:
                 user.update({
                     key[5:]: value
-                    for key, value in membership.iteritems()
+                    for key, value in membership.items()
                     if key.startswith('user_')
                 })
                 break
@@ -437,5 +457,32 @@ class CloseIO(object):
         try:
             self.find_user_id(email)
             return True
-        except:
+        except Exception:
             return False
+
+    @parse_response
+    @handle_errors
+    def create_lead_export(self, query='*', format='json', fields=(),
+                           include_activities=False, include_smart_fields=False):
+
+        args = dict(
+            format=format,
+            type='leads',
+            query=query,
+        )
+
+        if include_activities:
+            args['include_activities'] = include_activities
+
+        if include_smart_fields:
+            args['include_smart_fields'] = include_smart_fields
+
+        if fields:
+            args['fields'] = list(fields)
+
+        return self._api.export.lead.post(args)
+
+    @parse_response
+    @handle_errors
+    def get_export(self, id):
+        return self._api.export(id).get()
