@@ -1,11 +1,5 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
 from django.dispatch import Signal
 
-
 closeio_event = Signal(providing_args=["instance", "model", "event"], use_caching=True)
 
 closeio_create = Signal(providing_args=["instance", "model"], use_caching=True)
