@@ -1,8 +1,7 @@
-from django.conf.urls import patterns, url
+from django.conf.urls import url
 
 from .views import CloseIOWebHook
 
-urlpatterns = patterns(
-    '',
+urlpatterns = [
     url(r'^$', CloseIOWebHook.as_view(), name='closeio_webhook'),
-)
+]
