@@ -1,20 +1,16 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import json
 import logging
 
 from django.http import HttpResponse, HttpResponseBadRequest
 from django.utils.decorators import method_decorator
-from django.views.generic import View
+from django.utils.encoding import force_text
 from django.views.decorators.csrf import csrf_exempt
+from django.views.generic import View
 
 from closeio import utils
 
 from . import signals
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -25,7 +21,7 @@ class CloseIOWebHook(View):
 
     def post(self, request, *args, **kwargs):
         try:
-            query = json.loads(request.body)
+            query = json.loads(force_text(request.body))
         except (ValueError, TypeError):
             logger.exception("CloseIO webhook request could not be parsed.")
             return HttpResponseBadRequest()
