@@ -1,18 +1,13 @@
-# coding=utf-8
-from __future__ import (
-    absolute_import, division, print_function, unicode_literals
-)
-
 import contextlib
-import json
 import types
 from datetime import date, datetime, time
 from functools import wraps
 
 import dateutil.parser
+from six import string_types, text_type
 from slumber.exceptions import SlumberBaseException
 
-from closeio.exceptions import CloseIOError
+from closeio.exceptions import CloseIOError, RateLimitError
 
 
 @contextlib.contextmanager
@@ -24,22 +19,29 @@ def convert_errors():
         raise
 
     except SlumberBaseException as e:
-        if hasattr(e, 'content'):
-            try:
-                error_info = json.loads(e.content)
-                if 'error' in error_info:
-                    error_message = error_info['error']
-                else:
-                    error_message = e.content
-            except ValueError:
-                error_message = unicode(e.content)
-        else:
-            error_message = unicode(e)
+        if hasattr(e, 'response'):
 
-        raise CloseIOError(error_message, e)
+            try:
+                error_info = e.response.json()
+                if e.response.status_code == 429:
+                    raise RateLimitError(**error_info['error'])
+                error_message = error_info['error']
+            except (ValueError, KeyError):
+                error_message = e.response.text
+
+            request = e.response.request
+            request_data = 'url: {}\nbody: {}'.format(
+                request.url,
+                request.body,
+            )
+        else:
+            error_message = text_type(e)
+            request_data = ''
+
+        raise CloseIOError(error_message, e, request_data)
 
     except Exception as e:
-        raise CloseIOError(unicode(e), e)
+        raise CloseIOError(text_type(e), e, '')
 
 
 def handle_errors(func):
@@ -57,7 +59,7 @@ class Item(dict):
         self.__dict__ = self
 
 
-def parse(value):
+def parse(value):  # NoQA
     try:
         return Item({
             key: parse(value)
@@ -72,7 +74,7 @@ def parse(value):
             for item in value
         )
 
-    if not isinstance(value, basestring):
+    if not isinstance(value, string_types):
         try:
             return [
                 parse(item)
@@ -107,7 +109,7 @@ def convert(value):
     except AttributeError:
         pass
 
-    if not isinstance(value, basestring):
+    if not isinstance(value, string_types):
         try:
             return [
                 convert(item)
@@ -141,6 +143,13 @@ def paginate(func, *args, **kwargs):
         with convert_errors():
             response = func(*args, **kwargs)
 
+        if not isinstance(response, dict):
+            raise CloseIOError(
+                'close.io response is not a dict, '
+                'so most likely could not be parsed. \n'
+                'body "{}"'.format(response)
+            )
+
         for item in response['data']:
             yield item
 
